import os
import psutil
import logging
import subprocess

PROCESS_NAME = "RDPAgent.exe"

logger = logging.getLogger(__name__)


def start_agent():
    current_file_path = os.path.dirname(os.path.realpath(__file__))
    cmd = [os.path.join(current_file_path, "agent", PROCESS_NAME)]
    logger.info(f"Attempting to execute command '{' '.join(cmd)}'")
    p = subprocess.Popen(cmd, creationflags=0x00000008)
    logger.info(f"Successfully started the process, PID: {p.pid}")


def restart_agent():
    logger.info(f"Restarting the process {PROCESS_NAME}")
    kill_processes(PROCESS_NAME)
    start_agent()


def check_agent_running():
    logger.info(f"Checking if {PROCESS_NAME} is running")
    if not get_processes(PROCESS_NAME):
        logger.info(f"No {PROCESS_NAME} found, attempting to start it")
        start_agent()


def kill_agent():
    kill_processes(PROCESS_NAME)


def get_processes(name: str) -> list:
    process_list = []
    logger.info([p.name() for p in psutil.process_iter(attrs=["name"])])
    for p in psutil.process_iter(attrs=["name"], ad_value="unknow"):
        if p.name() == name:
            logger.info(f"Found process '{name}': {p}")
            process_list.append(p)
    return process_list


def kill_processes(name: str) -> bool:
    for process in get_processes(name):
        # At first, attempt a SIGTERM
        logger.info(f"Attempting to cleanly terminate '{process}'")
        try:
            process.terminate()
            gone, alive = psutil.wait_procs([process], timeout=10)
        except psutil.AccessDenied:
            logger.error(
                f"Could not kill {process}. Is it running as a different user than {os.getlogin()}?"
            )
            return False
        if alive:
            try:
                # If the process is alive after 10 seconds, SIGKILL
                logger.info(f"Attempting to forcefully terminate '{process}'")
                process.kill()
            except psutil.NoSuchProcess:
                pass
        gone, alive = psutil.wait_procs([process], timeout=10)
        if alive:
            # If we get here something is wrong, we cannot kill the process
            logger.error(f"Could not kill the process '{process}'")
            return False
        else:
            logger.info(f"Terminated the process '{process}'")
            return True
