import os
import time
import json
import atexit
import logging
import subprocess

from ruxit.api.selectors import HostSelector
from ruxit.api.results_builder import PluginProperty
from ruxit.api.base_plugin import BasePlugin, MEAttribute

from agent_controller import check_agent_running, restart_agent, kill_agent

current_file_path = os.path.dirname(os.path.realpath(__file__))
logger = logging.getLogger(__name__)


class RDPPlugin(BasePlugin):
    def initialize(self, **kwargs):
        self.update_config()

        restart_agent()
        atexit.register(kill_agent)

    def query(self, **kwargs):
        self._report_property("RDP Extension version", kwargs["json_config"]["version"])

        check_agent_running()

        self.report_vda_properties()

    def report_vda_properties(self):
        rdp_file_path = os.path.join(current_file_path, "agent", "config", "rdp.json")
        try:
            with open(rdp_file_path, "r") as f:
                properties = json.load(f)
            if "TerminalProtocol" in properties.keys():
                self._report_property("RDP Version", properties.get("TerminalProtocol"))                    
        except Exception as ex:
            logger.warning(f"Could not process vda.json: {ex}")

    def update_config(self):
        config = {
            "rum_enabled": self.config.get("rum_enabled"),
            "rum_backoff": self.config.get("rum_backoff"),
            "mint_enabled": self.config.get("mint_enabled"),
            "mint_frequency": int(self.config.get("mint_frequency")),
            "mint_login_failures": self.config.get("mint_login_failures"),
            "openkit_beacon_url": self.config.get("ok_beacon_url"),
            "openkit_application_id": self.config.get("ok_application_id"),
            "openkit_verify_certificates": self.config.get("ok_verify_certificates"),
            "openkit_use_oneagent_endpoints": self.config.get("ok_use_oneagent_endpoints"),
            "log_level": self.config.get("log_level"),
            "last_updated": int(time.time()*1000)
        }
        config_file = os.path.join(current_file_path, "agent", "config", "config.json")
        with open(config_file, "w") as f:
            logger.info("Writing config.json")
            json.dump(config, f, indent=2)

    def _report_property(self, key: str, value: str):
        self.results_builder.add_property(PluginProperty(key, value, HostSelector(), MEAttribute.CUSTOM_HOST_METADATA))